import portpicker
from fastapi.responses import Response
import requests
from tensorboard import notebook


class ColabInternalFetcher:
    tb_port = 0

    def start_tensorboard_colab(self, ipython, tb_logs_dir, logfile=None):
        tb_port = portpicker.pick_unused_port()
        if logfile is not None:
            ipython.system_raw(f"tensorboard --port {tb_port} --logdir {tb_logs_dir} >{logfile} 2>&1 &")
        else:
            ipython.system_raw(f"tensorboard --port {tb_port} --logdir {tb_logs_dir} &")
        self.tb_port = tb_port
        return tb_port

    def start_tensorboard(self, tb_logs_dir, logfile=None):
        tb_port = portpicker.pick_unused_port()
        notebook.start(f"--port {tb_port} --logdir {tb_logs_dir}")
        self.tb_port = tb_port
        return tb_port

    def _fetch(self, url):
        response = requests.get(url)
        content_type = response.headers.get("Content-Type")
        return Response(content=response.content, media_type=content_type)
        # try:
        #     response = requests.get(url)
        #     if response.status_code == 200:
        #         content_type = response.headers.get("Content-Type")
        #         print(content_type)
        #         if "application/json" in content_type:
        #             data = response.json()
        #         else:
        #             # レスポンスがバイナリデータの場合
        #             base64_encoded_data = base64.b64encode(response.content)
        #             data = base64_encoded_data.decode("utf-8")  # Base64文字列にする

        #         response = ColabInternalFetchResponse(
        #             status="OK",
        #             url=url,
        #             message="",
        #             data=data,
        #         )
        #     else:
        #         response = ColabInternalFetchResponse(
        #             status="NG",
        #             url=url,
        #             message=f"http response code is not 200, {response.status_code}.",
        #             data=None,
        #         )
        # except requests.exceptions.RequestException as e:
        #     response = ColabInternalFetchResponse(
        #         status="NG",
        #         url=url,
        #         message=f"Exception, {e}.",
        #         data=None,
        #     )

        # json_compatible_item_data = jsonable_encoder(response)
        # return JSONResponse(content=json_compatible_item_data)

    def get_runs(self):
        url = f"http://localhost:{self.tb_port}/data/runs"
        return self._fetch(url)

    def get_scalars_tags(self):
        url = f"http://localhost:{self.tb_port}/data/plugin/scalars/tags"
        return self._fetch(url)

    def get_scalars_scalars(self, run: str, tag: str):
        url = f"http://localhost:{self.tb_port}/data/plugin/scalars/scalars?run={run}&tag={tag}"
        return self._fetch(url)

    def get_images_tags(self):
        url = f"http://localhost:{self.tb_port}/data/plugin/images/tags"
        return self._fetch(url)

    def get_images_images(self, run: str, tag: str, sample: int):
        url = f"http://localhost:{self.tb_port}/data/plugin/images/images?run={run}&tag={tag}&sample={sample}"
        return self._fetch(url)

    def get_images_individualImage(self, blob_key: str):
        url = f"http://localhost:{self.tb_port}/data/plugin/images/individualImage?blob_key={blob_key}"
        return self._fetch(url)

    def get_audio_tags(self):
        url = f"http://localhost:{self.tb_port}/data/plugin/audio/tags"
        return self._fetch(url)

    def get_audio_audio(self, run: str, tag: str, sample: int):
        url = f"http://localhost:{self.tb_port}/data/plugin/audio/audio?run={run}&tag={tag}&sample={sample}"
        return self._fetch(url)

    def get_audio_individualAudio(self, blob_key: str):
        url = f"http://localhost:{self.tb_port}/data/plugin/audio/individualAudio?blob_key={blob_key}"
        return self._fetch(url)



from fastapi import APIRouter
from typing import Callable, Dict, Any
from fastapi import Response
from dataclasses import dataclass

from colab_easy_ui.data.Response import FunctionInfoResponse, FunctionTaskStatusResponse, JsonApiFuncInfo

from fastapi.encoders import jsonable_encoder
from fastapi.responses import JSONResponse


@dataclass
class JsonApiFunc:
    id: str
    display_name: str
    method: str
    path: str
    func: Callable[[Dict[str, Any]], Dict[str, Any]]


class Functions:
    def __init__(self):
        self.status_store = {}

    def register_functions(self, funcs: list[JsonApiFunc]):
        self.functions = funcs

        router = APIRouter()
        for func in funcs:
            router.add_api_route(func.path, func.func, methods=[func.method])
        router.add_api_route("/functions", self.get_function_info, methods=["GET"])
        router.add_api_route("/functions_set_task_status", self.set_task_status, methods=["GET"])
        router.add_api_route("/functions_del_task_status", self.del_task_status, methods=["GET"])
        router.add_api_route("/functions_get_task_status", self.get_task_status, methods=["GET"])

        self.router = router

    def set_task_status(self, task_id: str, status: str, data: str):
        self.status_store[task_id] = {"status": status, "data": data}
        print(self.status_store)

    def del_task_status(self, task_id: str):
        del self.status_store[task_id]

    def get_task_status(self, task_id: str):
        data = FunctionTaskStatusResponse(
            status="OK",
            message="",
            task_id=task_id,
            task_status=self.status_store[task_id]["status"] if hasattr(self.status_store, task_id) else "not found",
            task_status_data=self.status_store[task_id]["data"],
        )
        json_compatible_item_data = jsonable_encoder(data)
        return JSONResponse(content=json_compatible_item_data)

    def get_function_info(self):
        if self.functions is None:
            data = Response(
                status="error",
                message="No functions registered.",
            )
            return JSONResponse(content=jsonable_encoder(data))

        functions: list[JsonApiFuncInfo] = []
        for f in self.functions:
            f_info = JsonApiFuncInfo(
                id=f.id,
                display_name=f.display_name,
                method=f.method,
                path=f.path,
            )
            functions.append(f_info)

        data = FunctionInfoResponse(
            status="ok",
            message="",
            functions=functions,
        )

        json_compatible_item_data = jsonable_encoder(data)
        return JSONResponse(content=json_compatible_item_data)



import fire
from colab_easy_ui.ColabEasyUI import ColabEasyUI, JsonApiFunc
from fastapi.encoders import jsonable_encoder
from fastapi.responses import JSONResponse
import uuid
import requests


def run_server():
    c = ColabEasyUI.get_instance()
    port = c.port
    c.enable_file_uploader("upload", {"abc": "voice.zip"})
    c.enable_colab_internal_fetcher()
    tb_port = c.colabInternalFetcher.start_tensorboard("trainer/amitaro/logs", "TB_LOG")
    print("Tensorflow port:::", tb_port)

    def get_info():
        # UUIDを作成
        uuid_str = str(uuid.uuid4())

        # httpのgetを"http://localhost:8000/test"に対して実行
        response = requests.get(f"http://localhost:{port}/functions_set_task_status?task_id={uuid_str}&status=running&data=running")

        response = requests.get(f"http://localhost:{port}/functions_set_task_status?task_id={uuid_str}&status=fin&data=running")

        try:
            data = {
                "status": "ok",
                "uuid": uuid_str,
                "description": "easy-file-uploader-py created by wok!",
            }

            json_compatible_item_data = jsonable_encoder(data)
            return JSONResponse(content=json_compatible_item_data)
        except Exception as e:
            data = {
                "status": "error",
                "message": str(e),
                "description": "easy-file-uploader-py created by wok!",
            }
            print(data)
            return JSONResponse(content=json_compatible_item_data)

    c.register_functions(
        [
            JsonApiFunc("idid1", "function1", "GET", "/test", get_info),
            JsonApiFunc("idid2", "function2", "POST", "/test2", get_info),
        ]
    )
    c.mount_static_folder("/front2", "frontend/dist")
    port = c.start()
    print(port)


def main():
    fire.Fire(run_server)



from dataclasses import dataclass
from typing import TypeAlias, Literal

from typing import Dict

# from typing import Union, Dict

RESPONSE_STATUS: TypeAlias = Literal[
    "OK",
    "NG",
]


@dataclass
class Response:
    status: RESPONSE_STATUS
    message: str


@dataclass
class JsonApiFuncInfo:
    id: str
    display_name: str
    method: str
    path: str


@dataclass
class FunctionInfoResponse(Response):
    functions: list[JsonApiFuncInfo]


@dataclass
class FunctionTaskStatusResponse(Response):
    task_id: str
    task_status: str
    task_status_data: str


# @dataclass
# class ColabInternalFetchResponse(Response):
#     url: str
#     data: Dict[str, Union[str, int, float]] | None


@dataclass
class EasyFileUploaderResponse(Response):
    allowed_filenames: Dict[str, str] | None = None
